#include "kem_int_flows.h"
#include "sampling.h"
#include "parallel_hash.h"
#include "gf2x.h"

ret_t encrypt(OUT ct_t *ct,
              IN const pk_t *pk,
              IN const seed_t *seed,
              IN const split_e_t *splitted_e)
{
    DEFER_CLEANUP(padded_r_t m = {0}, padded_r_cleanup);
    DEFER_CLEANUP(dbl_pad_ct_t p_ct, dbl_pad_ct_cleanup);

    // Pad the public key
    pad_pk_t p_pk = {0};
    VAL(p_pk[0]) = PTRV(pk)[0];
    VAL(p_pk[1]) = PTRV(pk)[1];

    DMSG("    Sampling m.\n");
    GUARD(sample_uniform_r_bits(VAL(m).raw, seed, NO_RESTRICTION));

    EDMSG("m:  "); print((uint64_t*)VAL(m).raw, R_BITS);

    DMSG("    Calculating the ciphertext.\n");

    GUARD(gf2x_mod_mul(p_ct[0].u.qw, m.u.qw, p_pk[0].u.qw));
    GUARD(gf2x_mod_mul(p_ct[1].u.qw, m.u.qw, p_pk[1].u.qw));

    DMSG("    Addding Error to the ciphertext.\n");

    GUARD(gf2x_add(VAL(p_ct[0]).raw, VAL(p_ct[0]).raw, PTRV(splitted_e)[0].raw, R_SIZE));
    GUARD(gf2x_add(VAL(p_ct[1]).raw, VAL(p_ct[1]).raw, PTRV(splitted_e)[1].raw, R_SIZE));

    // Copy the data outside
    PTRV(ct)[0] = VAL(p_ct[0]);
    PTRV(ct)[1] = VAL(p_ct[1]);

    EDMSG("c0: "); print((uint64_t*)PTRV(ct)[0].raw, R_BITS);
    EDMSG("c1: "); print((uint64_t*)PTRV(ct)[1].raw, R_BITS);

    return SUCCESS;
}

ret_t calc_pk(OUT pk_t *pk,
              IN const seed_t *g_seed,
              IN const pad_sk_t p_sk)
{
    //PK is dbl padded because modmul require scratch space for the multiplication result
    dbl_pad_pk_t p_pk = {0};

    //Must intialized padding to zero!!
    DEFER_CLEANUP(padded_r_t g = {0}, padded_r_cleanup);
    GUARD(sample_uniform_r_bits(VAL(g).raw, g_seed, MUST_BE_ODD));
    
    EDMSG("g:  "); print((uint64_t*)VAL(g).raw, R_BITS);

    // Calculate (g0, g1) = (g*h1, g*h0)
    GUARD(gf2x_mod_mul(p_pk[0].u.qw, g.u.qw, p_sk[1].u.qw));
    GUARD(gf2x_mod_mul(p_pk[1].u.qw, g.u.qw, p_sk[0].u.qw));

    // Copy the data outside
    PTRV(pk)[0] = VAL(p_pk[0]);
    PTRV(pk)[1] = VAL(p_pk[1]);

    EDMSG("g0: "); print((uint64_t *) PTRV(pk)[0].raw, R_BITS);
    EDMSG("g1: "); print((uint64_t *) PTRV(pk)[1].raw, R_BITS);

    return SUCCESS;
}

// Generate the Shared Secret (K(e))
void get_ss(OUT ss_t *out, IN const e_t *e)
{
    DMSG("    Enter get_ss.\n");

    // Calculate the hash
    sha_hash_t hash = {0};
    parallel_hash(&hash, e->raw, sizeof(*e));

    // Truncate the final hash into K by copying only the LSBs
    memcpy(out->raw, hash.u.raw, sizeof(*out));

    secure_clean(hash.u.raw, sizeof(hash));
    DMSG("    Exit get_ss.\n");
}
