CC := gcc
CPP := g++

INC := -I${ROOT} -I${ROOT}/common -I${ROOT}/decode -I${ROOT}/prf -I${ROOT}/hash -I${ROOT}/gf2x

CFLAGS := -ggdb -O3 $(INC)

uname_m := $(shell uname -m)

ifeq ($(uname_m),aarch64)
  AARCH64 := 1
  USE_OPENSSL := 1
else
  ifeq ($(uname_m),x86)
    X86 := 1
    CFLAGS += -m32 -maes
  else
    ifeq ($(uname_m),x86_64)
      X86_64 := 1
      CFLAGS += -m64 -maes
    else
      $(error "Only x86_64, x86, and aarch64 platforms are currently supported.")
    endif
  endif
endif

ifdef RDTSC
    CFLAGS += -DRDTSC
endif

ifdef VERBOSE
    CFLAGS += -DVERBOSE=$(VERBOSE)
endif

ifdef BIKE_VER
    CFLAGS += -DBIKE_VER=$(BIKE_VER)
else
    ifeq ($(MAKECMDGOALS),)
        $(info using BIKE_VER=1 as default.)
        CFLAGS += -DBIKE_VER=1
    endif
endif

ifdef FIXED_SEED
    CFLAGS += -DFIXED_SEED=1
endif

ifdef NUM_OF_TESTS
    CFLAGS += -DNUM_OF_TESTS=$(NUM_OF_TESTS)
endif

    
ifndef AARCH64
    CFLAGS += -mno-red-zone 
endif
CFLAGS += -fvisibility=hidden -funsigned-char -Wall -Wextra -Werror -Wpedantic 
CFLAGS += -Wunused -Wcomment -Wchar-subscripts -Wuninitialized -Wshadow
CFLAGS += -Wwrite-strings -Wno-deprecated-declarations -Wno-unknown-pragmas -Wformat-security
CFLAGS += -Wcast-qual -Wunused-result

ifndef VERBOSE
CFLAGS += -Wcast-align
endif

#Avoiding GCC 4.8 bug
CFLAGS += -Wno-missing-braces -Wno-missing-field-initializers

CFLAGS += -std=c99

ifdef BATCH_SIZE
    CFLAGS += -DBATCH_SIZE=$(BATCH_SIZE)
endif

ifdef BIKE3_BANDWIDTH_OPT
    CFLAGS += -DBIKE3_BANDWIDTH_OPT
endif

ifdef AVX512
    CFLAGS += -mavx512f -DAVX512
    SUF = _avx512
else
    ifdef AVX2
        CFLAGS += -mavx2
        SUF = _avx2
    else
        PORTABLE = 1
        CFLAGS += -DPORTABLE
    endif
endif

ifdef CONSTANT_TIME
    CFLAGS += -DCONSTANT_TIME
endif

ifdef VALIDATE_CONSTANT_TIME
    CFLAGS += -DVALIDATE_CONSTANT_TIME
endif

ifdef LEVEL
    CFLAGS += -DLEVEL=$(LEVEL)
endif

ifdef USE_OPENSSL
    CFLAGS += -DUSE_OPENSSL
endif

ifdef OPENSSL
    #Add OpenSSL
    CFLAGS += -I$(OPENSSL)/include -L$(OPENSSL) -lcrypto -ldl -lpthread
endif

ifdef USE_NIST_RAND
    ifdef FIXED_SEED
        $(error cant have both FIXED_SEED and USE_NIST_RAND)
    endif

    ifndef OPENSSL
        $(error OPENSSL path variable is missing)
    endif

    #Turn on NIST_RAND
    CFLAGS += -DUSE_NIST_RAND=1
endif

CPPFLAGS = $(filter-out -std=c99, $(CFLAGS))
CPPFLAGS += -std=c++11