/***************************************************************************
* Additional implementation of "BIKE: Bit Flipping Key Encapsulation". 
* Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
*
* Written by Nir Drucker and Shay Gueron
* AWS Cryptographic Algorithms Group
* (ndrucker@amazon.com, gueron@amazon.com)
*
* The license is detailed in the file LICENSE.md, and applies to this file.
* ***************************************************************************/

#pragma once

#include "types.h"
#include "aes_ctr_prf.h"

#if BIKE_VER==1
ret_t encrypt(OUT ct_t *ct,
              IN const pk_t *pk,
              IN const seed_t *seed,
              IN const split_e_t *splitted_e);

ret_t calc_pk(OUT pk_t *pk,
              IN const seed_t *g_seed,
              IN const pad_sk_t p_sk);

void get_ss(OUT ss_t *out, IN const e_t *e);

#elif BIKE_VER==2
typedef struct batch_ctx_s
{
    uint32_t cnt;
    padded_r_t h0[BATCH_SIZE];
    compressed_idx_dv_t sk_wlist[BATCH_SIZE];
    r_t tmp1[BATCH_SIZE];
    r_t tmp0[BATCH_SIZE];
    r_t inv;
} batch_ctx_t;

ret_t encrypt(OUT ct_t *ct,
              IN const pk_t *pk,
              IN const split_e_t *splitted_e);

ret_t calc_pk(OUT pk_t *pk, IN OUT sk_t *sk);

void get_ss(OUT ss_t *out, IN const e_t *e);

#if BATCH_SIZE > 1
ret_t init_batch(IN OUT aes_ctr_prf_state_t *h_prf_state,
                 IN OUT batch_ctx_t *ctx);

ret_t get_batch_keys(OUT pk_t *pk, 
                     IN OUT sk_t *sk, 
                     IN OUT batch_ctx_t *ctx);
#endif

#elif BIKE_VER==3
ret_t encrypt(OUT ct_t *ct,
              IN const pk_t *pk,
              IN const split_e_t *splitted_e,
              IN const padded_r_t *e_extra);

ret_t calc_pk(OUT pk_t *pk,
              IN const seed_t *g_seed,
              IN OUT sk_t *sk);

void get_ss(OUT ss_t *out, IN const e_t *e, IN const r_t *e_extra);
#endif
