#!/bin/bash

# Abort the script if any command failed to return 0
set -e 
set -x

openssl_dir=`realpath ../openssl`

echo "Test BIKE non constant time with NTL and AVX2."

for bike_ver in "1" "2" "3"; do
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s RDTSC=1 AVX2=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
    done
done

echo "Test BIKE semi-constant time with NTL and AVX2."

for bike_ver in "1" "2" "3"; do
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s CONSTANT_TIME=1 RDTSC=1 AVX2=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
    done
done

echo "Test BIKE non constant time with OpenSSL and AVX2."

for bike_ver in "1" "2" "3"; do
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s RDTSC=1 OPENSSL=$openssl_dir AVX2=1 USE_OPENSSL=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
    done
done

echo "Test BIKE non constant time with NTL."

for bike_ver in "1" "2" "3"; do
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s RDTSC=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
    done
done

echo "Test BIKE non constant time with OpenSSL."

for bike_ver in "1" "2" "3"; do
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s RDTSC=1 OPENSSL=$openssl_dir USE_OPENSSL=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
    done
done

echo "Test BIKE semi-constant time with NTL."

for bike_ver in "1" "2" "3"; do
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s CONSTANT_TIME=1 RDTSC=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
    done
done

echo "Test BIKE semi-constant time with OpenSSL."

for bike_ver in "1" "2" "3"; do
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s CONSTANT_TIME=1 RDTSC=1 OPENSSL=$openssl_dir USE_OPENSSL=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
    done
done
