#!/bin/bash

# Abort the script if any command failed to return 0
set -e

if [ $# -eq 0 ]; then
    echo "No arguments supplied - please supply a test level.";
    exit
fi

test_num=$1
openssl_dir=`realpath ../openssl`

function check_constant_kats {
    # BIKE L1
    diff PQCkemKAT_BIKE_4670.rsp kats/BIKE1_L1.const.kat > /dev/null
    diff PQCkemKAT_BIKE_5941.rsp kats/BIKE2_L1.const.kat > /dev/null
    diff PQCkemKAT_BIKE_7628.rsp kats/BIKE3_L1.const.kat > /dev/null

    # BIKE L3
    diff PQCkemKAT_BIKE_8116.rsp kats/BIKE1_L3.const.kat > /dev/null
    diff PQCkemKAT_BIKE_10598.rsp kats/BIKE2_L3.const.kat > /dev/null
    diff PQCkemKAT_BIKE_13996.rsp kats/BIKE3_L3.const.kat > /dev/null

    # BIKE L5
    diff PQCkemKAT_BIKE_12364.rsp kats/BIKE1_L5.const.kat > /dev/null
    diff PQCkemKAT_BIKE_16458.rsp kats/BIKE2_L5.const.kat > /dev/null
    diff PQCkemKAT_BIKE_22244.rsp kats/BIKE3_L5.const.kat > /dev/null

    # Clean
    rm PQCkemKAT*
}

function check_kats {
    # Compare all
    # BIKE L1
    diff PQCkemKAT_BIKE_3606.rsp kats/BIKE1_L1.kat > /dev/null
    diff PQCkemKAT_BIKE_4877.rsp kats/BIKE2_L1.kat > /dev/null
    diff PQCkemKAT_BIKE_6572.rsp kats/BIKE3_L1.kat > /dev/null

    # BIKE L3
    diff PQCkemKAT_BIKE_6540.rsp kats/BIKE1_L3.kat > /dev/null
    diff PQCkemKAT_BIKE_9022.rsp kats/BIKE2_L3.kat > /dev/null
    diff PQCkemKAT_BIKE_12420.rsp kats/BIKE3_L3.kat > /dev/null

    # BIKE L5
    diff PQCkemKAT_BIKE_10276.rsp kats/BIKE1_L5.kat > /dev/null
    diff PQCkemKAT_BIKE_14370.rsp kats/BIKE2_L5.kat > /dev/null
    diff PQCkemKAT_BIKE_20156.rsp kats/BIKE3_L5.kat > /dev/null

    # Clean
    rm PQCkemKAT*
}

function check_kats_bike3_opt {
    # BIKE L3 OPT
    diff PQCkemKAT_BIKE_5225.rsp kats/BIKE3_L1.bandwidth.opt.kat > /dev/null
    diff PQCkemKAT_BIKE_9741.rsp kats/BIKE3_L3.bandwidth.opt.kat > /dev/null
    diff PQCkemKAT_BIKE_15671.rsp kats/BIKE3_L3.bandwidth.opt.kat > /dev/null

    # Clean
    rm PQCkemKAT*
}

# Print the instructios
set -x

##########################
# Test BIKE constant time.
##########################

if [ "$test_num" -lt "1" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            make -s clean; 
            make -s CONSTANT_TIME=1 USE_NIST_RAND=1 OPENSSL=$openssl_dir BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
            ./bin/main
        done
    done
    check_constant_kats
fi

####################################################
# Test BIKE constant time with USE_OPENSSL=1.
####################################################

if [ "$test_num" -lt "2" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            make -s clean; 
            make -s CONSTANT_TIME=1 USE_NIST_RAND=1 USE_OPENSSL=1 OPENSSL=$openssl_dir BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
            ./bin/main
        done
    done
    check_constant_kats
fi

####################################################
# Test BIKE constant time with AVX2=1.
####################################################

if [ "$test_num" -lt "3" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            make -s clean; 
            make -s CONSTANT_TIME=1 USE_NIST_RAND=1 AVX2=1 OPENSSL=$openssl_dir BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
            ./bin/main
        done
    done
    check_constant_kats
fi

###############################
# Test BIKE non constant time.
###############################

if [ "$test_num" -lt "4" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            make -s clean; 
            make -s USE_NIST_RAND=1 OPENSSL=$openssl_dir BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
        done
    done
    check_kats
fi

####################################################
# Test BIKE non constant time + USE_OPENSSL
####################################################

if [ "$test_num" -lt "5" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            make -s clean; 
            make -s USE_NIST_RAND=1 OPENSSL=$openssl_dir USE_OPENSSL=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
        ./bin/main
        done
    done
    check_kats
fi

####################################################
# Test BIKE non constant time + USE_OPENSSL + AVX2
####################################################

if [ "$test_num" -lt "6" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            make -s clean; 
            make -s USE_NIST_RAND=1 OPENSSL=$openssl_dir AVX2=1 USE_OPENSSL=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
            ./bin/main
        done
    done
    check_kats
fi

####################################################
# Test BIKE non constant time + AVX2
####################################################

if [ "$test_num" -lt "7" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            make -s clean; 
            make -s USE_NIST_RAND=1 OPENSSL=$openssl_dir AVX2=1 BIKE_VER=$bike_ver LEVEL=$level;  > /dev/null
            ./bin/main
        done
    done
    check_kats
fi

####################################################
# Test gcc and clang compilation only
####################################################

if [ "$test_num" -lt "8" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            for compiler in "gcc" "clang"; do
                make -s clean; 
                make -s BIKE_VER=$bike_ver LEVEL=$level CC=$compiler; 
                make -s clean; 
                make -s CONSTANT_TIME=1 BIKE_VER=$bike_ver LEVEL=$level CC=$compiler; 
                make -s clean; 
                make -s USE_OPENSSL=1 BIKE_VER=$bike_ver LEVEL=$level CC=$compiler; 
                make -s clean; 
                make -s CONSTANT_TIME=1 USE_OPENSSL=1 BIKE_VER=$bike_ver LEVEL=$level CC=$compiler; 
            done
        done
    done
fi

####################################################
# Test AVX512 
####################################################

if [ "$test_num" -lt "9" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            make -s clean; 
            make -s CONSTANT_TIME=1 AVX512=1 BIKE_VER=$bike_ver LEVEL=$level; 
            make -s clean; 
            make -s AVX512=1 BIKE_VER=$bike_ver LEVEL=$level; 
        done
    done
fi

####################################################
# Test compilation with verbose
####################################################

if [ "$test_num" -lt "10" ]; then
    # Compile all
    for bike_ver in "1" "2" "3"; do
        for level in "1" "3" "5"; do
            for compiler in "gcc" "clang"; do
                make -s clean; 
                make -s VERBOSE=3 CONSTANT_TIME=1 USE_OPENSSL=1 AVX2=1 BIKE_VER=$bike_ver LEVEL=$level; 
                make -s clean; 
                make -s VERBOSE=3 CONSTANT_TIME=1 AVX2=1 BIKE_VER=$bike_ver LEVEL=$level; 
                make -s clean; 
                make -s VERBOSE=3 BIKE_VER=$bike_ver LEVEL=$level CC=$compiler; 
            done
        done
    done
fi

####################################################
# Test BATCH + verbose
####################################################

if [ "$test_num" -lt "11" ]; then
    # Compile all
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s VERBOSE=3 CONSTANT_TIME=1 USE_OPENSSL=1 AVX2=1 BIKE_VER=2 LEVEL=$level BATCH_SIZE=10; 
        make -s clean; 
        make -s VERBOSE=3 CONSTANT_TIME=1 AVX2=1 BIKE_VER=2 LEVEL=$level BATCH_SIZE=10;
    done
fi

####################################################
# Test BIKE3 with optimized bandwidth
####################################################

if [ "$test_num" -lt "12" ]; then
    # Compile all
    for level in "1" "3" "5"; do
        make -s clean; 
        make -s USE_NIST_RAND=1 OPENSSL=$openssl_dir AVX2=1 BIKE_VER=3 BIKE3_BANDWIDTH_OPT=3 LEVEL=$level; 
        ./bin/main
    done
    check_kats_bike3_opt
fi